-- MainWindow.lua
-- Originally TheOneBagWindow.lua written by Rod, extended by Habna


import "HabnaPlugins.HugeBag.settingsWindow"; -- LUA settings file
import "HabnaPlugins.HugeBag.functionsWindow"; -- LUA functions file
import "HabnaPlugins.HugeBag.optionsWindow"; -- LUA options file


local resources = {
	  Item = { Background = "HabnaPlugins/HugeBag/Resources/slotBackground.tga" },
};

LoadSettings();

if ShowTitle then
	frmMain = class( Turbine.UI.Lotro.Window ); --Window with skin & title
else
	frmMain = class( Turbine.UI.Window ); --Window with no skin & no title
end

function frmMain()
	if ShowSkin then
		wHugeBag = Turbine.UI.Lotro.Window();
		--Turbine.UI.Lotro.Window.Constructor( wHugeBag ); --Window with skin & title
	else
		wHugeBag = Turbine.UI.Window();
		--Turbine.UI.Window.Constructor( wHugeBag ); --Window with no skin no title
	end

	--**v Set HugeBag window v**
	wHugeBag:SetSize( WinWidth, WinHeight );
	wHugeBag:SetPosition( WinLocX, WinLocY );
	wHugeBag:SetWantsKeyEvents( true );
	if zOrder then wHugeBag:SetZOrder( 1 );	end --1 Always on top / 0 Not always on top
	wHugeBag:Activate();
	wHugeBag.cursor = nil;
	--**^
	--**v Check for the Reloader & Unloader module v**
	Turbine.PluginManager.RefreshAvailablePlugins();
	loaded_plugins = Turbine.PluginManager.GetLoadedPlugins();

	HBChecker = Turbine.UI.Control();
	HBChecker:SetWantsUpdates( true );
		
	HBChecker.Update = function( sender, args )
		for k,v in pairs(loaded_plugins) do
			if v.Name == "HugeBag Reloader" then
				Turbine.PluginManager.UnloadScriptState( 'HugeBagReloader' );
				wHugeBag:SetVisible( WasVisible );
				--write("Bug detected, HugeBag was reloaded!");
				bfound = true;
				--break;
			end
			if v.Name == "HugeBag Unloader" then
				Turbine.PluginManager.UnloadScriptState( 'HugeBagUnloader' );
				--break;
			end
		end
		HBChecker:SetWantsUpdates( false );
		if not bfound then wHugeBag:SetVisible( StartVisible ); end
		WasVisible = wHugeBag:IsVisible();
		SaveSettings();
	end
	--**^
	--**v Skin v**
	HBSkin = Turbine.UI.Control();
	HBSkin:SetParent( wHugeBag );
	HBSkin:SetPosition( 0, 0 );
	HBSkin:SetSize( WinWidth, WinHeight );
	HBSkin:SetBackColor( none );
	HBSkin:SetVisible( not ShowSkin );

	HBSkin.MouseMove = function( sender, args )
		wHugeBag.MouseMove( sender, args );
	end

	HBSkin.MouseDown = function( sender, args )
		wHugeBag.MouseDown( sender, args );
	end

	HBSkin.MouseUp = function( sender, args )
		wHugeBag.MouseUp( sender, args );
	end

	HBSkin.MouseLeave = function( sender, args )
		wHugeBag.MouseLeave( sender, args );
	end
	--**^
	--**v Lock Label v**
	LockLabel = Turbine.UI.Control();
	LockLabel:SetParent( wHugeBag );
	LockLabel:SetPosition( 23, 0 );
	LockLabel:SetSize( WinWidth - 50, 40 );
	LockLabel:SetVisible( true );
	LockLabel:SetZOrder( 1 );
	--LockLabel:SetBackColor( white );
	LockLabel.Enable = not LockPosition;

	LockLabel.MouseClick = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Right ) then ReloadHugeBag(); end
	end
	
	LockLabel.MouseMove = function( sender, args )
		CornerClick = "Title";
		wHugeBag.MouseMove( sender, args );
	end
	
	LockLabel.MouseDown = function( sender, args )
		if not LockPosition then
			CornerClick = "Title";
			wHugeBag.MouseDown( sender, args );
		end
	end

	LockLabel.MouseUp = function( sender, args )
		wHugeBag.MouseUp( sender, args );
	end

	LockLabel.MouseLeave = function( sender, args )
		CornerClick = "";
		wHugeBag.MouseLeave( sender, args );
	end
	--**^
	-- **v Sort button v**
	SortButton = Turbine.UI.Button();
	SortButton:SetParent( wHugeBag );
	SortButton:SetText( "Sort" );
	SortButton:SetSize( 45, 12 );
	SortButton:SetForeColor( grey );
	SortButton:SetOutlineColor( black );
	SortButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SortButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	SortButton:SetZOrder( 2 );
	SortButton:SetVisible( ShowButton );

	SortButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Sort();
		else
			FromButton = "Sort";
			if ( SortButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); else Mix(); end
		end
	end

	SortButton.MouseHover = function(sender, args)
		SortButton:SetForeColor( green );
	end

	SortButton.MouseLeave = function(sender, args)
		SortButton:SetForeColor( grey );
	end
	-- **^
	-- **v Search button v**
	SearchButton = Turbine.UI.Button();
	SearchButton:SetParent( wHugeBag );
	SearchButton:SetText( "Search" );
	SearchButton:SetSize( 45, 12 );
	SearchButton:SetForeColor( grey );
	SearchButton:SetOutlineColor( black );
	SearchButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	SearchButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	SearchButton:SetZOrder( 2 );
	SearchButton:SetVisible( ShowButton );

	SearchButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Searching();
		else
			FromButton = "Search";
			if ( SearchButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); end
		end
	end

	SearchButton.MouseHover = function(sender, args)
		SearchButton:SetForeColor( green );
	end

	SearchButton.MouseLeave = function(sender, args)
		SearchButton:SetForeColor( grey );
	end
	--**^
	--**v Merge button on bottom bar v**
	MergeButton = Turbine.UI.Button();
	MergeButton:SetParent( wHugeBag );
	MergeButton:SetText( "Merge" );
	MergeButton:SetSize( 45, 12 );
	MergeButton:SetOutlineColor( black );
	MergeButton:SetForeColor( grey );
	MergeButton:SetFontStyle( Turbine.UI.FontStyle.Outline );
	MergeButton:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
	MergeButton:SetZOrder( 2 );
	MergeButton:SetVisible( ShowButton );

	MergeButton.MouseClick = function(sender, args)
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			Merge();
		else
			FromButton = "Merge";
			if ( MergeButton:GetLeft() == MidPos ) then buttonmenu:ShowMenu(); end
		end
	end

	MergeButton.MouseHover = function(sender, args)
		MergeButton:SetForeColor( green );
	end

	MergeButton.MouseLeave = function(sender, args)
		MergeButton:SetForeColor( grey );
	end
	--**^
	-- **v Set the itemlistbox v**
	wHugeBag.itemListBox = Turbine.UI.ListBox();
	wHugeBag.itemListBox:SetParent( wHugeBag );
	--wHugeBag.itemListBox:SetReverseFill( ReverseFill );
	wHugeBag.itemListBox:SetOrientation( Turbine.UI.Orientation.Vertical );
	if HorizontalOrientation then wHugeBag.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal ); end
	wHugeBag.itemListBox:SetBackColor( Turbine.UI.Color( bcAlpha, bcRed, bcGreen, bcBlue ) );
	wHugeBag.itemListBox:SetAllowDrop( true );

	wHugeBag.itemListBox.DragDrop = function( sender, args )
		--write("drag/drop");
		local shortcut = args.DragDropInfo:GetShortcut();
		local destinationItemControl = wHugeBag.itemListBox:GetItemAt( args.X, args.Y );
		
		if ( destinationItemControl ~= nil ) then -- ** if nil, then it's out of range, so don't do anything **
			local destinationIndex = wHugeBag.itemListBox:IndexOfItem( destinationItemControl );
			Backpack:PerformShortcutDrop(shortcut, destinationIndex, Turbine.UI.Control.IsShiftKeyDown() );
		end
	end
	-- **^
	-- **v Set the scrollbar v**
	wHugeBag.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	wHugeBag.itemListBoxScrollBar:SetParent( wHugeBag );	
	-- **^
	-- **v Search box v**
	SearchInput = Turbine.UI.Lotro.TextBox();
	SearchInput:SetParent( wHugeBag );
	SearchInput.Text = "";
	SearchInput:SetSize( 150, 20 );
	SearchInput:SetMultiline( false );
	SearchInput:SetFont( Turbine.UI.Lotro.Font.TrajanPro14 );
	SearchInput:SetText( SearchInput.Text );
	SearchInput:SetVisible( false );
	SearchInput:SetZOrder( 2 );
	
	SearchInput.FocusGained = function()
		SearchInput:SetWantsUpdates( true );
	end
	
	SearchInput.FocusLost = function()
		SearchInput:SetWantsUpdates( false );
		SearchInput:SetText( "" );
		SearchInput:SetVisible( false );
		if ShowButton then
			SortButton:SetVisible( true );
			SearchButton:SetVisible( true );
			MergeButton:SetVisible( true );
			Search();
			AllVisible = true;
		end
	end
	
	SearchInput.Update = function()
		Search();
	end
	-- **^
	-- **v Get player gameplay v**
	if pcall( Turbine.Gameplay.LocalPlayer.GetInstance ) then
		Backpack = Turbine.Gameplay.LocalPlayer.GetInstance():GetBackpack();
	else
		Backpack = Turbine.Gameplay.LocalPlayer():GetBackpack();
	end
	-- **^

	backpackSize = Backpack:GetSize();
	SetBackPack(); --> ** Set the backpack **
	
	wHugeBag.MouseDown = function( sender, args )
		if ( args.Button == Turbine.UI.MouseButton.Left ) then
			if CornerClick ~= "Title" then
				if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then
					CornerClick = "TopLeft";
					--wHugeBag.cursor:SetVisible( false );
					if wHugeBag:GetLeft() >= 204 then
						itemsmenu:ShowMenuAt(wHugeBag:GetLeft() - 200, wHugeBag:GetTop());
					else
						itemsmenu:ShowMenuAt(wHugeBag:GetLeft() + wHugeBag:GetWidth() + 5, wHugeBag:GetTop());
					end
				elseif ( ( args.X < 23 ) and ( args.Y > ( wHugeBag:GetHeight() - 30 ) ) ) then
					CornerClick = "BottomLeft";
				elseif ( args.X > ( wHugeBag:GetWidth() -30 ) ) and ( args.Y > ( wHugeBag:GetHeight() - 30 ) ) then
					CornerClick = "BottomRight";
				elseif ( args.X > 50 ) and ( args.X < ( wHugeBag:GetWidth() - 50 ) ) and ( args.Y > ( wHugeBag:GetHeight() - 12 ) ) then
					CornerClick = "BottomBorder";
				elseif ( args.X < 6 ) and ( args.Y > 60 ) and ( args.Y < ( wHugeBag:GetHeight() - 45 ) ) then
					CornerClick = "LeftBorder";
				elseif ( args.X > ( wHugeBag:GetWidth() - 6 ) ) and ( args.Y > 60 ) and ( args.Y < ( wHugeBag:GetHeight() - 45 ) ) then
					CornerClick = "RightBorder";
				end
			end
			ClicX, ClicY = args.X + 23, args.Y;
			dragStartX = args.X;
			dragStartY = args.Y;
			dragging = true;
		elseif ( args.Button == Turbine.UI.MouseButton.Right ) then
			if CornerClick ~= "Title" then
				if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then 
					--TopLeft
					frmOptions();
				elseif ( ( args.X < 23 ) and ( args.Y > ( wHugeBag:GetHeight() - 30 ) ) ) then
					--BottomLeft
					
				end
			end
		end
		--write(tostring(CornerClick));
	end
	
	wHugeBag.MouseUp = function( sender, args )
		dragging = false;
		OutOfBound = false;
		CornerClick = "";
		SaveSettings();
	end
	
	wHugeBag.MouseMove = function( sender, args )
		--write(tostring(CornerClick));
		if ( dragging ) then
			wHugeBag.MouseLeave( sender, args );
			WinWidth, WinHeight = wHugeBag:GetSize();
			WinLocX, WinLocY = wHugeBag:GetPosition();
			local screenWidth = Turbine.UI.Display:GetWidth();
			local screenHeight = Turbine.UI.Display:GetHeight();
			local tempY = WinHeight + ( args.Y - dragStartY );

			if CornerClick == "BottomRight" then
				if not LockSize then
					-- **v Check horizontal v**
					local tempX = WinWidth + ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( wHugeBag:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					-- Prevent window to go out of bound (Right side)
					if ( WinLocX + tempX ) > screenWidth then
						wHugeBag:SetSize( ( screenWidth - WinLocX ), WinHeight );
					else
						wHugeBag:SetSize( tempX, tempY );
						dragStartX = dragStartX + (args.X - dragStartX);
						dragStartY = dragStartY + (args.Y - dragStartY);
					end
					-- **^
				end
			elseif CornerClick == "BottomLeft" then
				if not LockSize then
					-- **v Check horizontal v**
					local tempX = WinWidth - ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( wHugeBag:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					--Prevent window to go out of bound (Left side)
					if WinLocX < 0 then
						dragging = false;
						wHugeBag:SetPosition( 0, WinLocY );
					else
						wHugeBag:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
						wHugeBag:SetSize( tempX, tempY );
						dragStartY = dragStartY + ( args.Y - dragStartY );
					end
					-- **^
				end
			elseif CornerClick == "Title" then
				local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
				wHugeBag:SetPosition( mouseX - ClicX, mouseY - ClicY);
			elseif CornerClick == "BottomBorder" then
				if not LockSize then
					if tempY < 125 then -- Minimum height
						wHugeBag:SetHeight( 125 );
					else
						wHugeBag:SetHeight( tempY );
					end
					dragStartY = dragStartY + (args.Y - dragStartY);
				end
			elseif CornerClick == "LeftBorder" then
				if not LockSize then
					local tempX = WinWidth - ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( wHugeBag:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					if WinLocX < 0 then --Prevent window to go out of bound (Left side)
						dragging = false;
						wHugeBag:SetPosition( 0, WinLocY );
					elseif wHugeBag:GetWidth() <= 218 then
						dragging = false;
						wHugeBag:SetSize( 219, WinHeight );
					else
						wHugeBag:SetPosition( WinLocX + ( args.X - dragStartX ), WinLocY );
						wHugeBag:SetSize( tempX, WinHeight );
					end
				end
			elseif CornerClick == "RightBorder" then
				if not LockSize then
					local tempX = WinWidth + ( args.X - dragStartX );
					if not ButtonAtLeft then
						if ( wHugeBag:GetWidth() >= 218 ) then
							ButtonsLeft();
							ButtonsRight();
						end
					end
					if ( WinLocX + tempX ) > screenWidth then -- Prevent window to go out of bound (Right side)
						wHugeBag:SetSize( ( screenWidth - WinLocX ), WinHeight );
					else
						wHugeBag:SetSize( tempX, WinHeight );
						dragStartX = dragStartX + (args.X - dragStartX);
					end
				end
			end
			--[[ Need to go out of bound when no skin on HugeBag
			-- **v Check vertical v**
			-- Prevent window to go out of bound (Bottom)
			if ( WinLocY + tempY ) > screenHeight then
				wHugeBag:SetSize( WinWidth, ( screenHeight - WinLocY ) );
			end
			-- **^
			]]
			PerformLayout();
		else
			if CornerClick == "Title" then
				ShowTitleResizeCursor();
			else
				if ( args.X < 23 and args.Y > 15 and args.Y < 50 )then
					-- "TopLeft"
					wHugeBag.cursor = leftrightclicCursor;
					local mouseX, mouseY = Turbine.UI.Display.GetMousePosition();
					wHugeBag.cursor:SetPosition( mouseX - wHugeBag.cursor.xOffset, mouseY - wHugeBag.cursor.yOffset);
					wHugeBag.cursor:SetVisible( true );
				elseif ( ( args.X < 23 ) and ( args.Y > ( wHugeBag:GetHeight() - 30 ) ) ) then
					-- "BottomLeft"
					ShowResizeCursor();
				elseif ( args.X > ( wHugeBag:GetWidth() - 30 ) ) and ( args.Y > ( wHugeBag:GetHeight() - 30 ) ) then
					-- "BottomRight"
					ShowResizeCursor();
				elseif ( args.X > 50 ) and ( args.X < ( wHugeBag:GetWidth() - 50 ) ) and ( args.Y > ( wHugeBag:GetHeight() - 12 ) ) then
					-- Bottom border
					ShowUpDownCursor();
				elseif ( args.X < 6 ) and ( args.Y > 60 ) and ( args.Y < ( wHugeBag:GetHeight() - 45 ) ) then
					-- Left border
					ShowLeftRightCursor();
				elseif ( args.X > ( wHugeBag:GetWidth() - 6 ) ) and ( args.Y > 60 ) and ( args.Y < ( wHugeBag:GetHeight() - 45 ) ) then
					--Right border
					ShowLeftRightCursor();
				else
					wHugeBag.MouseLeave( sender, args );
				end
			end
		end
	end

	wHugeBag.MouseLeave = function( sender, args )
		if wHugeBag.cursor ~= nil then
			wHugeBag.cursor:SetVisible( false );
			wHugeBag.cursor = nil;
		end
	end
	
	wHugeBag.Unload = function( sender, args )
		
	end

	wHugeBag.Closing = function( sender, args ) -- Function for the Upper right X icon
		WasVisible = false;
		SaveSettings();
	end
	
	wHugeBag.PositionChanged = function(args)
		WinLocX, WinLocY = wHugeBag:GetPosition()
		SaveSettings();
	end
	
	wHugeBag.KeyDown = function( sender, args )
		if ( args.Action == 145 ) then -- **v Hide HugeBag if ESC key is press v** (Turbine.UI.Lotro.Action.Escape)
			if ( HideWEsc ) then
				wHugeBag:SetVisible( false );
				WasVisible = false;
				SaveSettings();
			end
		end

		if ( args.Action == 268435635 ) then -- **v Hide HugeBag if F12 key is press v**
			if wHugeBag:IsVisible() then wHugeBag:SetVisible( false ); end
			F12Press = not F12Press;
			if not F12Press then wHugeBag:SetVisible( WasVisible ); end
		end
		
		-- **v Show HugeBag if any key bound is press v**
		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 )
		then
			if not F12Preshs then -- **v show HugeBag only if F12 key was not press v**
				wHugeBag:SetVisible( not wHugeBag:IsVisible() )
				WasVisible = wHugeBag:IsVisible();
				SaveSettings();
			end
		end
	end
	
	Backpack.SizeChanged = function( sender, args )
		SetBackPack();
	end

	--Backpack.ItemAdded = function( sender, args )
	HBItemAddedHandler = function( sender, args )
		--write("item added to index " .. args.Index);
		bAdd = true;
		AddTime = Turbine.Engine.GetGameTime();
		--write("AddTime: " .. tostring(AddTime));
		wHugeBag.items[args.Index]:SetItem( Backpack:GetItem( args.Index ) );
		PrintTitle();
	end
	
	-- Workaround for the ItemRemoved
	ItemRemovedTimer = Turbine.UI.Control();
	
	ItemRemovedTimer.Update = function( sender, args )
		ItemRemovedTimer:SetWantsUpdates( false );
		PrintTitle();
	end
	--

	--Backpack.ItemRemoved = function( sender, args )
	HBItemRemovedHandler = function( sender, args )
		--write("item removed from index " .. args.Index);
		--write("bAdd: " .. tostring(bAdd) .. " IsSorting: " .. tostring(IsSorting) .. " IsMixing: " .. tostring(IsMixing));
		if bAdd and not IsSorting and not IsMixing and not IsMerging then
			RemTime = Turbine.Engine.GetGameTime();
			--write("RemTime: " .. tostring(RemTime));
			elapsed = RemTime - AddTime;
			--write("elapsed: " .. tostring(elapsed));
			if elapsed == 0 then
				bGo = true;
				--write("HugeBag will be reloaded!");
			end
		end
		bAdd = false;

		wHugeBag.items[args.Index]:SetItem( nil );
		--wHugeBag.items[args.Index]:SetItem( Backpack:GetItem( args.Index ) ); -- Original line
		ItemRemovedTimer:SetWantsUpdates( true ); --wHugeBag:PrintTitle();

		if bGo then ReloadHugeBag(); end
	end

	--Backpack.ItemMoved = function( sender, args )
	HBItemMovedHandler = function( sender, args )
		--write("item moved from index " .. args.OldIndex .. " to " .. args.NewIndex);
		wHugeBag.items[args.OldIndex]:SetItem( Backpack:GetItem( args.OldIndex ) );
		wHugeBag.items[args.NewIndex]:SetItem( Backpack:GetItem( args.NewIndex ) );
	end

	-- register callbacks
	AddCallback(Backpack, "ItemAdded", HBItemAddedHandler)
	AddCallback(Backpack, "ItemRemoved", HBItemRemovedHandler)
	AddCallback(Backpack, "ItemMoved", HBItemMovedHandler)
end

--**v Set the backpack v**
function SetBackPack()
	wHugeBag.items = nil;
	itemsCtl = nil;
	itemsLbl = nil;
	wHugeBag.itemListBox:ClearItems();

	wHugeBag.items = {};
	itemsCtl = {};
	itemsLbl = {};

	for i = 1, backpackSize do
		-- Item background
		itemsCtl[i] = Turbine.UI.Control();
		itemsCtl[i]:SetSize( 35, 35 );
		itemsCtl[i]:SetBackground( resources.Item.Background );
		itemsCtl[i]:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		-- Slot num in background
		itemsLbl[i] = Turbine.UI.Label();
		itemsLbl[i]:SetParent( itemsCtl[i] );
		itemsLbl[i]:SetSize( 35, 35 );
		itemsLbl[i]:SetPosition( 0, 0 );
		itemsLbl[i]:SetText( i );
		itemsLbl[i]:SetForeColor( verylightgrey );
		itemsLbl[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		itemsLbl[i]:SetBlendMode( Turbine.UI.BlendMode.Overlay );

		-- Item
		wHugeBag.items[i] = Turbine.UI.Lotro.ItemControl( Backpack:GetItem( i ) );
		wHugeBag.items[i]:SetParent( itemsCtl[i] );

		wHugeBag.itemListBox:AddItem( itemsCtl[i] );
	end

	PerformLayout();
	PrintTitle();
end
-- **^
-- **v Layout of the items in the listbox v**
function PerformLayout()
	WinWidth, WinHeight = wHugeBag:GetSize();
	WinLocX, WinLocY = wHugeBag:GetPosition();
	itemWidth, itemHeight = wHugeBag.items[1]:GetSize();
	
	listWidth = WinWidth - 37;
	listHeight = WinHeight - 52;

	wHugeBag.itemListBox:SetSize( listWidth, listHeight );
	wHugeBag.itemListBox:SetPosition( InitXPos, InitYPos );

	wHugeBag.itemListBox:SetOrientation( Turbine.UI.Orientation.Vertical );
	if HorizontalOrientation then wHugeBag.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal ); end
	
	if wHugeBag.itemListBox:GetOrientation() == 0 then -- 0 => Horizontal /  1=> Vertical
		itemsPerRow = listWidth / itemWidth;
		--if ReverseFill then
			
		--else
			wHugeBag.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
			wHugeBag.itemListBox:SetVerticalScrollBar( wHugeBag.itemListBoxScrollBar );

			wHugeBag.itemListBoxScrollBar:SetPosition( WinWidth - 20, 40 );
			wHugeBag.itemListBoxScrollBar:SetSize( 10, listHeight - 15 );
		--end
	else
		itemsPerRow = listHeight / itemHeight;
		--if ReverseFill then

		--else
			wHugeBag.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Horizontal );
			wHugeBag.itemListBox:SetHorizontalScrollBar( wHugeBag.itemListBoxScrollBar );

			wHugeBag.itemListBoxScrollBar:SetPosition( wHugeBag.itemListBox:GetLeft(), WinHeight - 23);
			wHugeBag.itemListBoxScrollBar:SetSize( listWidth, 10 );
		--end
	end
	
	wHugeBag.itemListBox:SetMaxItemsPerLine( itemsPerRow );
	
	if ( wHugeBag:GetWidth() <= 218 ) then wHugeBag:SetSize( 218, wHugeBag:GetHeight()); end -- Limit to minimum width

	LockLabel:SetSize(WinWidth - 50, 33 ); --> ** Resize the LockLabel with HugeBag width **
	HBSkin:SetSize( WinWidth, WinHeight ); --> ** Resize the HBSkin with HugeBag width an height **

	if ( ButtonAtTop ) then
		TopPos = 23;
	else
		TopPos = WinHeight - 16;
	end
	SortButton:SetPosition( SortLeft,  TopPos ); --> ** Replace 'Sort' button with HugeBag height **
	SearchButton:SetPosition( SearchLeft,  TopPos ); --> ** Replace 'Search' button with HugeBag height **
	MergeButton:SetPosition( MergeLeft, TopPos ); --> ** Replace 'Merge' button with HugeBag height **
	SearchInput:SetPosition( LeftPos,  TopPos - 7 ); --> ** Replace 'Search Box' with HugeBag height **
end
-- **^
-- **v Print the title with the free/used/max info v**
function PrintTitle()
	if ShowSlotsInfo then
		local freeslots = 0;
		
		for i = 1, backpackSize do
			if ( Backpack:GetItem( i ) == nil ) then freeslots = freeslots + 1; end
		end
		
		local usedslots = backpackSize - freeslots;

		if WinWidth < 265 then
			wHugeBag:SetText( freeslots .. "/" .. usedslots .. "/" .. backpackSize );
		elseif WinWidth < 315 then
			wHugeBag:SetText( "F: " .. freeslots .. "  U: " .. usedslots .. "  M: " .. backpackSize );
		elseif WinWidth < 365 then
			wHugeBag:SetText( "HB -- F: " .. freeslots .. "  U: " .. usedslots .. "  M: " .. backpackSize );
		elseif WinWidth < 430 then
			wHugeBag:SetText( "HugeBag -- F: " .. freeslots .. "  U: " .. usedslots .. "  M: " .. backpackSize );
		else
			wHugeBag:SetText( "HugeBag -- Free: " .. freeslots .. "  Used: " .. usedslots .. "  Max: " .. backpackSize );
		end
	else
		wHugeBag:SetText( "HugeBag" );
	end
	IsMoving = false;
end
-- **^